;      8xxtarg.ss   Revision 2.3  1/24/97
;
; This software was written by Symbios Logic Inc. to develop and test new
; products.  Symbios Logic assumes no liability for its use.  This software
; is released to the public domain to illustrate certain programming
; techniques for the 53c8xx chips in target mode.
;
; Revision 2.3 1/24/97  Scott Rawlings
; Inhanced performance.
;
; Revision 2.2 2/12/96  Scott Rawlings
; Fixed major problem with disconnect logic on write commands.
;
; Revision 2.1 11/13/95  Scott Rawlings
; Added support for SCSI-1 initiators that select without attention.
;
; Revision 2.0 10/31/95  Scott Rawlings
; Streamlined SCRIPTS for faster operation.  Changed use of memory move
; instruction for use with user options that manipulate upper memory.  Added
; seperate section of SCRIPTS to be used when stopped or busy to free up main
; code area for faster operation.
;
; Revision 1.0 8/23/95  Scott Rawlings
; Initial release.

;ABSOLUTE DECLARATIONS
ABSOLUTE read_access_medium		= 0x00
ABSOLUTE write_access_medium		= 0x01
ABSOLUTE last_write_disconnect	= 0x02
ABSOLUTE seek_command				= 0x03
ABSOLUTE set_up_synch_neg			= 0x04
ABSOLUTE set_up_wide_neg			= 0x05
ABSOLUTE non_handled_msg	    	= 0x06
ABSOLUTE bad_extended_msg			= 0x07
ABSOLUTE message_sent           	= 0x08
ABSOLUTE request_sense_command	= 0x09
ABSOLUTE inquiry_command			= 0x0a
ABSOLUTE read_capacity_command	= 0x0b
ABSOLUTE start_stop_command		= 0x0c
ABSOLUTE format_unit					= 0x0d
ABSOLUTE send_diagnostic			= 0x0e
ABSOLUTE command_aborted			= 0x0f
ABSOLUTE illegal_cmd					= 0x10
ABSOLUTE got_SIGP						= 0x11
ABSOLUTE done_with_copy				= 0x12
ABSOLUTE got_selected				= 0x13
ABSOLUTE done_with_busy_command	= 0x14

EXTERN count
EXTERN source_address
EXTERN destination_address

TABLE table_indirect \
	msg_out_buf = 1{??}, \
	cmd_buf	= 12{??}, \
	synch_neg_msg_out = 2{??}, \
	wide_neg_msg_out = 1{??}, \
	neg_msg_in = {0x01, 0x03, 0x01, 0x19, 0x08}, \
	stat_buf = {0x02}, \
	identify_msg_in_buf = {0x80},\
	msg_in_buf = 1{??}, \
	data_buf = 512{??}, \
	save_pointers = {0x02}, \
	disconnect_msg = {0x02, 0x04}, \
	selector_id = ID{0x33, 0x07, 0x00, 0x00}, \
	sense_data_buf = {0x00, 0x00, 0x06, 0x00, 0x00, 0x00, \
			  0x00, 0x0a, 0x00, 0x00, 0x00, 0x00, \
			  0x29, 0x00, 0x00, 0x00, 0x00, 0x00}, \
	inquiry_data_buf = {0x00, 0x00, 0x02, 0x00, 0x1f, 0x00, \
				 0x00, 0x10, 0x20, 0x20, 0x20, 0x20, \
				 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, \
				 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, \
				 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, \
				 0x20, 0x20, 0x20, 0x20, 0x20, 0x20}, \
	capacity_data_buf = {0x00, 0x80, 0x02, 0x00}



;***********************************************************************
; ENTRY declarations
ENTRY wait_select
ENTRY msg_out_phase
ENTRY tur
ENTRY stopped_busy_tur
ENTRY request_sense
ENTRY read_return
ENTRY read_reconnect
ENTRY write_return
ENTRY write_reconnect
ENTRY synch_wide_neg_return
ENTRY msg_in_phase
ENTRY inquiry
ENTRY read_capacity
ENTRY stopped_busy_wait_select
ENTRY copy_data

wait_select:
	wait select rel(SIGP_set) ;wait to be selected
	jump rel(command_phase), if not atn ;SCSI-1 initiator support
	move from msg_out_buf, with msg_out ;get message byte
	move sfbr to scratchb0 ;save the identify message
	call rel(msg_out_phase), if atn ;stay in message if atn still active

command_phase:
	move from cmd_buf, with cmd ;get SCSI command
	move scntl1 & 0xdf to scntl1 ;turns on the halt on parity error or atn

	jump rel(read), if 0x08 ;jump to set up read (6-byte read)
	int write_access_medium, if 0x0a ;interrupt to set up write (6-byte write)
	int seek_command, if 0x0b ;interrupt to perform seek
	int seek_command, if 0x2b ;interrupt to perform seek
	jump rel(read), if 0x28 ;jump to set up read (10-byte read)
	int write_access_medium, if 0x2a ;interrupt to set up write (10-byte write)

	jump rel(tur), if 0x00 ;jump to test unit ready
	int request_sense_command, if 0x03 ;interrupt to set up request sense command
	int inquiry_command, if 0x12 ;interrupt to set up inquiry command
	int read_capacity_command, if 0x25 ;interrupt to set up read capacity command

	int start_stop_command, if 0x1b ;interrupt to set up start/stop unit command

	jump rel(tur), if 0x2f ;verify command, go to tur
	jump rel(reserve_unit), if 0x16 ;jump to reserve unit
	jump rel(release_unit), if 0x17 ;jump to release unit
	int send_diagnostic, if 0x1d ;interrupt to set up send diagnostic
	int format_unit, if 0x04 ;interrupt to set up format unit
	int illegal_cmd ;interrupt on any other command

msg_out_phase:
	return, if not atn ;return if atn gone
	move from msg_out_buf, with msg_out ;get message byte
	jump rel(extended_msg), if 0x01 ;jump if extended message
	jump rel(abort), if 0x06 ;jump if abort message
	jump rel(msg_out_phase), if 0x08	;jump back if nop message
	int non_handled_msg ;interrupt if can't handle message

extended_msg:
	int bad_extended_msg, if not atn	;if atn gone, extended message was bad
	move from msg_out_buf, with msg_out	;get next message byte
	int bad_extended_msg, if not atn ;if atn gone, extended message was bad
	move from msg_out_buf, with msg_out	;this byte shows type of message
	jump rel(synch_neg), if 0x01 ;0x01 is a synchronous negotiation message
	jump rel(wide_neg), if 0x03 ;0x03 is a wide negotiation message
	int bad_extended_msg ;interrupt on any other type

synch_neg:
	move from synch_neg_msg_out, with msg_out	;move in the period and offset
	int set_up_synch_neg ;interrupt to set up synchronous and message

wide_neg:
	move from wide_neg_msg_out, with msg_out ;move in the width
	int set_up_wide_neg ;interrupt to set up answer

synch_wide_neg_return:
	move from neg_msg_in, with msg_in ;move out our answer to the negotiation
	jump rel(command_phase), if not atn	;jump to command_phase if atn gone
	jump rel(msg_out_phase)	;jump to msg_out_phse if atn sill active

msg_in_phase:
	move from msg_in_buf, with msg_in ;move a message to the initiator
	int message_sent ;interrupt to determine what to do next

tur:
	move from stat_buf, with status ;send out status byte
	move from msg_in_buf, with msg_in ;send out message byte
	move 0x20 to scntl1 ;turns off the halt on parity error or atn
	disconnect ;disconnect from the SCSI bus
	intfly ;interrupt to signal end of process
	jump rel(wait_select)

stopped_busy_tur:
	move from stat_buf, with status ;send out status byte
	move from msg_in_buf, with msg_in ;send out message byte
	move 0x20 to scntl1 ;turns off the halt on parity error or atn
	disconnect ;disconnect from the SCSI bus
	move scratcha1 to sfbr ;get the busy flag
	int done_with_busy_command, if 0x01 ;if busy, interrupt to continue
	intfly ;interrupt to signal end of process
	jump rel(stopped_busy_wait_select)

request_sense:
	move from sense_data_buf, with data_in ;move the sense data from the buffer
	move scratcha2 to sfbr ;get the stopped/busy flag
	jump rel(tur) if 0x00 ;go to the appropriate status and message phases
	jump rel(stopped_busy_tur)

inquiry:
	move from inquiry_data_buf, with data_in ;move out inquiry data
	move scratcha2 to sfbr ;get the stopped/busy flag
	jump rel(tur) if 0x00 ;go to the appropriate status and message phases
	jump rel(stopped_busy_tur)

read_capacity:
	move from capacity_data_buf, with data_in	;move out read capacity data
	move scratcha2 to sfbr ;get the stopped/busy flag
	jump rel(tur) if 0x00 ;go to the appropriate status and message phases
	jump rel(stopped_busy_tur)

read:
	move scratchb0 to sfbr ;get identify message
	jump rel(read_disconnect) if 0x40 and mask 0xbf	;jump disconnect if
																	;disconnects are allowed
	int read_access_medium ;interrupt to read data from medium

read_return:
	move from data_buf, with data_in	;move the data out from the buffer
	move scratchb1 to sfbr ;get the 'finished' flag
	jump rel(tur) if 0x00 ;jump to status and message if tranfer done
	move scratchb0 to sfbr ;get identify message
	jump rel(read_disconnect) if 0x40 and mask 0xbf	;jump to disconnect if
																	;disconnects are allowed
	move from save_pointers, with msg_in ;move out the save pointers message
	call rel(msg_out_phase) if atn ;jump to message out if atn active
	int read_access_medium ;interrupt to access medium

read_disconnect:
	move from disconnect_msg, with msg_in ;move out the disconnect message
	call rel(msg_out_phase) if atn ;jump to message out if atn active
	move 0x20 to scntl1 ;turns off the halt on parity error or atn
	disconnect ;disconnect from the bus
	move 0x10 to istat ;set the semaphore bit to say we are disconnected
	int read_access_medium ;interrupt to read data from medium

read_reconnect:
	move 0x00 to istat ;clear the semaphore bit
	reselect from selector_id, rel(alt_got_selected) ;reselect the initiator
	move scntl1 & 0xdf to scntl1 ;turns on the halt on parity error or atn
	move from identify_msg_in_buf, with msg_in ;move in identify message
	jump rel(read_return) ;jump to send data

write_return:
	move from data_buf, with data_out ;move the data into the buffer
	move scratchb0 to sfbr ;get identify message
	jump rel(write_disconnect) if 0x40 and mask 0xbf ;jump to disconnect if
																	 ;disconnects allowed
	move scratchb1 to sfbr ;get the 'finished' flag
	jump rel(tur) if 0x10 ;jump to status and message if transfer done
	move from save_pointers, with msg_in ;move out the save pointers message
	call rel(msg_out_phase) if atn ;jump to message out if atn active
	int write_access_medium	;interrupt to read data from medium

write_disconnect:
	move from disconnect_msg, with msg_in ;move out the disconnect message
	call rel(msg_out_phase) if atn ;jump to message out if atn active
	move 0x20 to scntl1 ;turns off the halt on parity error or atn
	disconnect ;disconnect from the bus
	move 0x10 to istat ;set the semaphore bit to say we are disconnected
	move scratchb1 to sfbr ;get the 'finished' flag
	int last_write_disconnect if 0x10 ;special interrupt after last data phase
	int write_access_medium	;interrupt to read data from medium

write_reconnect:
	move 0x00 to istat ;clear the semaphore bit
	reselect from selector_id, rel(alt_got_selected) ;reselect the initiator
	move scntl1 & 0xdf to scntl1 ;turns on the halt on parity error or atn
	move from identify_msg_in_buf, with msg_in ;move in identify message
	move scratchb1 to sfbr ;get the 'finished' flag
	jump rel(tur) if 0x00 ;jump to status and message if transfer done
	jump rel(write_return) ;jump to get data

reserve_unit:
	move 0x01 to scratchb2 ;set 'reserved' in reservation flag
	move ssid & 0x7f to sfbr ;get the ID of who reserved us
	move sfbr to scratchb3 ;move ID into storage buffer
	jump rel(tur) ;go to status and message

release_unit:
	move 0x00 to scratchb2 ;set 'not reserved' in reservation flag
	jump rel(tur) ;go to status and message

abort:
	move 0x20 to scntl1 ;turns off the halt on parity error or atn
	disconnect ;go to bus free
	int command_aborted ;int to notify driver that command was aborted

stopped_busy_wait_select:
	wait select rel(SIGP_set) ;wait to be selected
	move from msg_out_buf, with msg_out ;get message byte
	call rel(msg_out_phase), if atn ;stay in message if atn still active

	move from cmd_buf, with cmd ;get SCSI command
	move scntl1 & 0xdf to scntl1 ;turns on the halt on parity error or atn

	jump rel(stopped_busy_tur), if 0x00 ;jump to test unit ready
	int request_sense_command, if 0x03 ;interrupt to set up request sense command
	int inquiry_command, if 0x12 ;interrupt to set up inquiry command
	int read_capacity_command, if 0x25 ;interrupt to set up read capacity command

	move sfbr to scratcha3 ;save the first byte of the command
	move scratcha1 to sfbr ;get the busy flag
	jump rel(stopped_busy_tur), if 0x01 ;if busy, go right to status and message

	move scratcha3 to sfbr ;restore the first byte of the command
	int start_stop_command, if 0x1b ;interrupt to set up start/stop unit command

	jump rel(stopped_busy_tur) ;go to status and message for any other command

alt_got_selected:
	int got_selected ;interrupt because got selected during reselect attempt

SIGP_set:
	int got_SIGP ;interrupt because got SIGP

copy_data:
	move memory count, source_address, destination_address ;memory move to write
																			 ;SCRIPTS RAM and to
																			 ;transfer data to and
																			 ;from upper memory
	int done_with_copy ;signal completion of memory move

